import { IterableOptions } from './storage/Database';
export interface IndexRequest {
    previousEntry?: any;
    pendingProcesses?: number[];
    deleted?: boolean;
    sources?: Set<any>;
    version: number;
    triggers?: Set<any>;
    previousValues?: Map;
    value: {};
    by?: any;
    resolveOnCompletion?: Function[];
}
export declare const Index: {
    ({ Source }: {
        Source: any;
    }): {
        new (): {
            [x: string]: any;
            version: number;
            averageConcurrencyLevel: number;
            clearCache(): void;
            valueOf(): any;
            resetCache(): void;
        };
        whenProcessingComplete: Promise<any>;
        whenCommitted: Promise<any>;
        indexingProcess: Promise<any>;
        eventLog: any[];
        queuedUpdateArrays: any[];
        derivedFrom(...sources: (Function | {})[]): void;
        forValue(id: any, transition: any): any;
        forQueueEntry(id: any): any;
        indexEntry(id: any, transition: any): Promise<{
            commit: () => any;
        }>;
        pendingEvents: Map<any, any>;
        normalizeEntries(entries: any): any[];
        log(...args: any[]): void;
        warn(...args: any[]): void;
        updated(event: any, by: any): void;
        queueUpdates(eventSources: any): void;
        processQueuedUpdates(): Promise<void>;
        sendUpdates(eventSources: any): void;
        get(id: any): any;
        getIndexedKeys(id: any): any;
        getIndexedValues(range: IterableOptions): any;
        _getIndexedValues(range: IterableOptions, returnFullKeyValue?: boolean): any;
        /**
        * Indexing function, that defines the keys and values used in the indexed table.
        * This should be implemented by Index subclasses, and should be safe/functional
        * method with referential integrity (always returns the same results with same inputs),
        * as it is used to determine key/values on both addition and removal of entities.
        * @param data The object to be indexed
        * @return The return value can be an array of objects, where each object has a `key` and a `value`. It can only be an array of simple strings or numbers, if it is merely keys that need to be indexed, or even be a just a string (or number), if only a single key should be indexed
        **/
        indexBy(data: {}, sourceKey: string | number | boolean): Array<{
            key: string | number;
            value: any;
        } | string | number> | IterableIterator<any> | string | number;
        whenUpdatedInContext(context?: any): any;
        getInstanceIdsAndVersionsSince(version: any): any;
        openDatabase(): boolean;
        initializeData(): Promise<void>;
        getIdsFromKey(key: any): any;
        updateDBVersion(): void;
        resumeQueue(): any;
        clearEntries(set: any): Promise<any>;
        myEarliestPendingVersion: number;
        whenAllConcurrentlyIndexing?: Promise<any>;
        loadVersions(): void;
        readonly instances: any;
        getInstanceIds(range?: IterableOptions): any;
    };
    from(Source: any): {
        new (): {
            [x: string]: any;
            version: number;
            averageConcurrencyLevel: number;
            clearCache(): void;
            valueOf(): any;
            resetCache(): void;
        };
        whenProcessingComplete: Promise<any>;
        whenCommitted: Promise<any>;
        indexingProcess: Promise<any>;
        eventLog: any[];
        queuedUpdateArrays: any[];
        derivedFrom(...sources: (Function | {})[]): void;
        forValue(id: any, transition: any): any;
        forQueueEntry(id: any): any;
        indexEntry(id: any, transition: any): Promise<{
            commit: () => any;
        }>;
        pendingEvents: Map<any, any>;
        normalizeEntries(entries: any): any[];
        log(...args: any[]): void;
        warn(...args: any[]): void;
        updated(event: any, by: any): void;
        queueUpdates(eventSources: any): void;
        processQueuedUpdates(): Promise<void>;
        sendUpdates(eventSources: any): void;
        get(id: any): any;
        getIndexedKeys(id: any): any;
        getIndexedValues(range: IterableOptions): any;
        _getIndexedValues(range: IterableOptions, returnFullKeyValue?: boolean): any;
        /**
        * Indexing function, that defines the keys and values used in the indexed table.
        * This should be implemented by Index subclasses, and should be safe/functional
        * method with referential integrity (always returns the same results with same inputs),
        * as it is used to determine key/values on both addition and removal of entities.
        * @param data The object to be indexed
        * @return The return value can be an array of objects, where each object has a `key` and a `value`. It can only be an array of simple strings or numbers, if it is merely keys that need to be indexed, or even be a just a string (or number), if only a single key should be indexed
        **/
        indexBy(data: {}, sourceKey: string | number | boolean): Array<{
            key: string | number;
            value: any;
        } | string | number> | IterableIterator<any> | string | number;
        whenUpdatedInContext(context?: any): any;
        getInstanceIdsAndVersionsSince(version: any): any;
        openDatabase(): boolean;
        initializeData(): Promise<void>;
        getIdsFromKey(key: any): any;
        updateDBVersion(): void;
        resumeQueue(): any;
        clearEntries(set: any): Promise<any>;
        myEarliestPendingVersion: number;
        whenAllConcurrentlyIndexing?: Promise<any>;
        loadVersions(): void;
        readonly instances: any;
        getInstanceIds(range?: IterableOptions): any;
    };
};
declare const KeyIndex_base: any;
export declare class KeyIndex extends KeyIndex_base {
    version: number;
    averageConcurrencyLevel: number;
    static whenProcessingComplete: Promise<any>;
    static whenCommitted: Promise<any>;
    static indexingProcess: Promise<any>;
    static eventLog: any[];
    static queuedUpdateArrays: any[];
    static derivedFrom(...sources: (Function | {})[]): void;
    static forValue(id: any, transition: any): any;
    static forQueueEntry(id: any): any;
    static indexEntry(id: any, transition: any): Promise<{
        commit: () => any;
    }>;
    static pendingEvents: Map<any, any>;
    static normalizeEntries(entries: any): any[];
    static log(...args: any[]): void;
    static warn(...args: any[]): void;
    static updated(event: any, by: any): void;
    static queueUpdates(eventSources: any): void;
    static processQueuedUpdates(): Promise<void>;
    static sendUpdates(eventSources: any): void;
    static get(id: any): any;
    static getIndexedKeys(id: any): any;
    static getIndexedValues(range: IterableOptions): any;
    static _getIndexedValues(range: IterableOptions, returnFullKeyValue?: boolean): any;
    /**
    * Indexing function, that defines the keys and values used in the indexed table.
    * This should be implemented by Index subclasses, and should be safe/functional
    * method with referential integrity (always returns the same results with same inputs),
    * as it is used to determine key/values on both addition and removal of entities.
    * @param data The object to be indexed
    * @return The return value can be an array of objects, where each object has a `key` and a `value`. It can only be an array of simple strings or numbers, if it is merely keys that need to be indexed, or even be a just a string (or number), if only a single key should be indexed
    **/
    static indexBy(data: {}, sourceKey: string | number | boolean): Array<{
        key: string | number;
        value: any;
    } | string | number> | IterableIterator<any> | string | number;
    static whenUpdatedInContext(context?: any): any;
    static getInstanceIdsAndVersionsSince(version: any): any;
    clearCache(): void;
    valueOf(): any;
    static openDatabase(): boolean;
    static initializeData(): Promise<void>;
    static getIdsFromKey(key: any): any;
    static updateDBVersion(): void;
    static resumeQueue(): any;
    static clearEntries(set: any): Promise<any>;
    static myEarliestPendingVersion: number;
    static whenAllConcurrentlyIndexing?: Promise<any>;
    static loadVersions(): void;
    resetCache(): void;
    static get instances(): any;
    static getInstanceIds(range?: IterableOptions): any;
}
export default Index;
