/// <reference types="node" />
import { Database, IterableOptions } from './storage/Database.js';
export declare const INVALIDATED_ENTRY: {
    state: string;
};
export declare const allStores: Map<any, any>;
export declare const ENTRY: unique symbol;
export interface IndexRequest {
    previousEntry?: any;
    pendingProcesses?: number[];
    deleted?: boolean;
    sources?: Set<any>;
    version: number;
    triggers?: Set<any>;
    fromValues?: Map;
    by?: any;
    resolveOnCompletion?: Function[];
}
export declare const PersistedBase: {
    new (): {
        [x: string]: any;
        setValue(value: any): void;
    };
    [x: string]: any;
    childStores: {
        forValue: Function;
        prepareCommit: Function;
        lastUpdate: number;
    }[];
    get(id: any, mode?: any): any;
    is(id: any, value: any, event: any): any;
    saveValue(id: any, entry: any, version?: any): any;
    reads: number;
    cachedReads: number;
    getInstanceIds(range: IterableOptions): any;
    entries(range: any): any;
    /**
    * Iterate through all instances to find instances since the given version
    **/
    getIdsFromKey(startKey: any): number[];
    dataVersionBuffer: Buffer;
    processKey: Buffer;
    lastIndexedVersion: any;
    remove(id: any, event?: any): any;
};
export declare class Persisted extends PersistedBase {
    db: any;
    static dbFolder: string;
    static clearAllData(): void;
    static set(id: any, value: any, event: any): any;
    patch(properties: any): any;
    put(value: any, event: any): any;
    static syncVersion: number;
}
export default Persisted;
export declare const Persistable: {
    new (id: any): {
        [x: string]: any;
        _cachedValue: any;
        _cachedVersion: number;
        _versions: any;
        version: number;
        db: Database;
        repetitiveGets: boolean;
        readonly staysUpdated: boolean;
        transform(source: any): void;
        delete(): any;
        reset(action: any): void;
        valueOf(mode: any): any;
        getValue(mode: any): any;
        gotValue(value: any): any;
        updated(event?: any, by?: any): any;
        notifies(target: any): any;
        stopNotifies(target: any): any;
        init(): any;
        cleanup(): any;
        exclusiveLock(executeWithLock: () => any): any;
    };
    [x: string]: any;
    DB: any;
    syncVersion: number;
    Cached: any;
    useWeakMap: boolean;
    whenWritten: Promise<any>;
    dbFolder: string;
    db: Database;
    updatingProcessConnection: {
        sendMessage(data: any): Promise<any>;
    };
    readonly defaultInstance: any;
    for(id: any): any;
    getByIds(ids: any): any;
    index(propertyName: string, indexBy?: (value: any, sourceKey: any) => any): any;
    relatesBy(foreignKey: string): {
        (): void;
        defineAs(propertyName: any, Parent: any): void;
    };
    relatedBy(foreignKey: string): {
        (): void;
        defineAs(propertyName: any, Parent: any): void;
    };
    cacheWith(properties: any): any;
    updatesRecorded(event: any): Promise<void> | Promise<[unknown, unknown, unknown, unknown, unknown, unknown, unknown, unknown, unknown, unknown]>;
    readonly ready: any;
    start(): any;
    clearAllData(): any;
    register(sourceCode?: {
        id?: string;
        version?: number;
    }): any;
    doesInitialization: any;
    initializeRootDB(): void;
    getStructureVersion(): number;
    openDatabase(): boolean;
    initialize(): any;
    doDataInitialization(): any;
    cleanupDeadProcessReference(pid: any, initializingProcess: any): any;
    reset(): Promise<void>;
    initializeData(): Promise<void>;
    instanceSetUpdated(event: any): void;
    clearEntryCache(id: any): void;
    invalidateEntry(id: any, event: any): void;
    update(id: any, event: any): any;
    updated(event?: any, by?: any): any;
    saveDBVersions(): void;
    updateDBVersion(): any;
    subscribedInstances: any;
    notifies(target: any): any;
    stopNotifies(target: any): any;
    whenUpdatedInContext(waitForIndexing: any): any;
    readonly instanceIds: any;
    tryForQueueEntry(id: any, action: any): any;
    queue: Map<any, IndexRequest>;
    processQueue(queue: any): Promise<void>;
    forQueueEntry(id: any): any;
    requestProcessing(nice: any, queue: any): any;
    openDB(store: any, options?: any): any;
    openChildDB(store: any, options?: any): any;
    removeUnusedDBs(): void;
    resumeQueue(): Promise<void>;
    resetAll(): any;
} & Secured;
declare const Cached_base: {
    new (): {
        [x: string]: any;
        setValue(value: any): void;
    };
    [x: string]: any;
    childStores: {
        forValue: Function;
        prepareCommit: Function;
        lastUpdate: number;
    }[];
    get(id: any, mode?: any): any;
    is(id: any, value: any, event: any): any;
    saveValue(id: any, entry: any, version?: any): any;
    reads: number;
    cachedReads: number;
    getInstanceIds(range: IterableOptions): any;
    entries(range: any): any;
    /**
    * Iterate through all instances to find instances since the given version
    **/
    getIdsFromKey(startKey: any): number[];
    dataVersionBuffer: Buffer;
    processKey: Buffer;
    lastIndexedVersion: any;
    remove(id: any, event?: any): any;
};
export declare class Cached extends Cached_base {
    allowDirectJSON: boolean;
    static sources: any[];
    static fetchAllIds: () => {}[];
    static get(id: any, mode?: any): any;
    static whenValueCommitted: Promise<any>;
    static runTransform(id: any, entry: any, mode: any): any;
    getValue(): any;
    is(value: any, event: any): this;
    static openChildDB(store: any, options: any): any;
    static updated(event: any, by?: any): any;
    static enqueue(id: any, event: any, previousEntry?: any): void;
    static resetAll(): Promise<any>;
    static invalidateEntry(id: any, event: any, by: any): void;
    static receiveRequest({ id, waitFor }: {
        id: any;
        waitFor: any;
    }): Promise<void>;
    static _version: number;
    static get version(): any;
    static set version(version: any);
    static returnsAsyncIterables: boolean;
    static from(...sources: Array<Function & {
        notifies: () => any;
        for: (id: any) => any;
        returnsAsyncIterables: boolean;
    }>): {
        new (): {
            [x: string]: any;
            readonly checkSourceVersions: boolean;
            allowDirectJSON: boolean;
            getValue(): any;
            is(value: any, event: any): any;
            setValue(value: any): void;
        };
        sources: any[];
        fetchAllIds: () => {}[];
        get(id: any, mode?: any): any;
        whenValueCommitted: Promise<any>;
        runTransform(id: any, entry: any, mode: any): any;
        openChildDB(store: any, options: any): any;
        updated(event: any, by?: any): any;
        enqueue(id: any, event: any, previousEntry?: any): void;
        resetAll(): Promise<any>;
        invalidateEntry(id: any, event: any, by: any): void;
        receiveRequest({ id, waitFor }: {
            id: any;
            waitFor: any;
        }): Promise<void>;
        _version: number;
        version: any;
        returnsAsyncIterables: boolean;
        from(...sources: (Function & {
            notifies: () => any;
            for: (id: any) => any;
            returnsAsyncIterables: boolean;
        })[]): any;
        derivedFrom(...sources: ({} | Function | Persisted)[]): void;
        getInstanceIds(range: any): any;
        updateDBVersion(): void;
        readonly whenProcessingComplete: Promise<any[]>;
        childStores: {
            forValue: Function;
            prepareCommit: Function;
            lastUpdate: number;
        }[];
        is(id: any, value: any, event: any): any;
        saveValue(id: any, entry: any, version?: any): any;
        reads: number;
        cachedReads: number;
        entries(range: any): any;
        /**
        * Iterate through all instances to find instances since the given version
        **/
        getIdsFromKey(startKey: any): number[];
        dataVersionBuffer: Buffer;
        processKey: Buffer;
        lastIndexedVersion: any;
        remove(id: any, event?: any): any;
    };
    static derivedFrom(...sources: Array<Persisted | Function | {}>): void;
    static getInstanceIds(range: any): any;
    static updateDBVersion(): void;
    static get whenProcessingComplete(): Promise<any[]>;
}
declare type PermissionCheck = (source: any, session: any, action: string, args: Array<any>) => boolean | string;
declare type Secured = {
    allow(...permissions: Array<PermissionCheck>): any;
};
export declare function secureAccess<T>(Class: T): T & Secured;
export declare namespace secureAccess {
    var checkPermissions: () => boolean;
}
export declare function getCurrentStatus(): {
    name: any;
    indexed: any;
    queueSize: any;
    size: any;
    state: any;
    concurrencyLevel: any;
}[];
export declare function configure(options: any): void;
export declare function writeCommonStructures(): any[];
export declare class Invalidated {
    constructor(version: any, processId?: any);
    version: number;
    value: number;
}
